/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.ldappasswd;

import java.io.ByteArrayOutputStream;
import javax.naming.ldap.ExtendedResponse;

/**
 * <p>The implementation for LDAP Password Modify Extended Operation response.
 * it just can contain the generated password if the request did not send
 * any new password field. Not all the servers supports generating passwords.
 * If the response did not return any password the password will be null.</p>
 *
 * @author rickyepoderi
 */
public class PasswordModifyExtendedResponse implements ExtendedResponse {

    // contants for values in the sequence
    protected static final byte TYPE_GENERATED_PASSWORD = (byte) 0x80;

    // value in the response
    private final String generatedPassword;

    /**
     * Empty constructor.
     */
    public PasswordModifyExtendedResponse() {
        this(null);
    }

    /**
     * Constructor passing the generated password.
     *
     * @param generatedPassword The generated password returned by the server.
     */
    public PasswordModifyExtendedResponse(String generatedPassword) {
        this.generatedPassword = generatedPassword;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getID() {
        return PasswordModifyExtendedRequest.PASSWORD_MODIFY_REQUEST_OID;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public byte[] getEncodedValue() {
        if (generatedPassword == null) {
            return null;
        }
        ByteArrayOutputStream buffer = new ByteArrayOutputStream(128);
        PasswordModifyExtendedRequest.encodeASN1OctetString(TYPE_GENERATED_PASSWORD, generatedPassword, buffer);
        return PasswordModifyExtendedRequest.encodeASN1Sequence(buffer.toByteArray());
    }

    /**
     * Getter for the generated password. If the server did not return any
     * password null will be returned.
     *
     * @return The password returned by the server or null
     */
    public String getGeneratedPassword() {
        return generatedPassword;
    }
}
