/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.ldappasswd;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Properties;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;
import org.apache.directory.server.core.integ.AbstractLdapTestUnit;
import org.apache.directory.server.core.integ.FrameworkRunner;
import org.apache.directory.server.core.annotations.CreateDS;
import org.apache.directory.server.core.annotations.CreatePartition;
import org.apache.directory.server.annotations.CreateLdapServer;
import org.apache.directory.server.annotations.CreateTransport;
import org.apache.directory.server.core.annotations.ApplyLdifFiles;
import org.apache.directory.server.ldap.handlers.extended.PwdModifyHandler;
import org.apache.directory.server.ldap.handlers.extended.StartTlsHandler;
import org.junit.Assert;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.junit.Test;

/**
 * <p>Test the extended operation using embedded ApacheDS. This implementation
 * does not return a generated password (the new password is compulsory).
 * Therefore that part cannot be tested.</p>
 *
 * @author rickyepoderi
 */
@RunWith(FrameworkRunner.class)
@CreateDS(name = "myDS",
        partitions = {
            @CreatePartition(name = "test", suffix = "dc=rickyepoderi,dc=es")
        })
@CreateLdapServer(
        transports = {@CreateTransport(protocol = "LDAP"), @CreateTransport(protocol = "LDAPS")},
        extendedOpHandlers = {StartTlsHandler.class, PwdModifyHandler.class}
)
@ApplyLdifFiles({"users.ldif"})
public class LdapPasswordTest extends AbstractLdapTestUnit {

    @Before
    public void installKeyStoreWithCertificate() {
        System.setProperty("javax.net.ssl.trustStore", getLdapServer().getKeystoreFile());
        System.setProperty("javax.net.ssl.trustStorePassword", "secret");
    }

    private LdapContext createConnection(String bindDN, String bindPwd) throws NamingException, UnknownHostException {
        Properties env = new Properties();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, "ldaps://" + InetAddress.getLocalHost().getHostAddress() + ":" + getLdapServer().getPortSSL());
        env.put(Context.SECURITY_PROTOCOL, "ssl");
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_PRINCIPAL, bindDN);
        env.put(Context.SECURITY_CREDENTIALS, bindPwd);
        env.put("com.sun.jndi.ldap.connect.pool", "false");
        return new InitialLdapContext(env, null);
    }

    private LdapContext createConnectionAdmin() throws NamingException, UnknownHostException {
        return createConnection("uid=admin,ou=system", "secret");
    }

    private void checkPassword(String bindDN, String bindPwd) throws NamingException, UnknownHostException {
        LdapContext ctx = createConnection(bindDN, bindPwd);
        ctx.close();
    }

    @Test
    public void test1ChangePasswordSelf() throws NamingException, UnknownHostException {
        LdapContext ctx = createConnection("cn=test1,ou=People,dc=rickyepoderi,dc=es", "pwd@test1");
        PasswordModifyExtendedRequest request = new PasswordModifyExtendedRequest(
                null, "pwd@test1", "changed@test1");
        PasswordModifyExtendedResponse response = (PasswordModifyExtendedResponse) ctx.extendedOperation(request);
        Assert.assertNull(response.getGeneratedPassword());
        checkPassword("cn=test1,ou=People,dc=rickyepoderi,dc=es", "changed@test1");
        ctx.close();
    }

    @Test
    public void test2ChangePasswordAdmin() throws NamingException, UnknownHostException {
        LdapContext ctx = createConnectionAdmin();
        PasswordModifyExtendedRequest request = new PasswordModifyExtendedRequest(
                "cn=test2,ou=People,dc=rickyepoderi,dc=es", "pwd@test2", "changed@test2");
        PasswordModifyExtendedResponse response = (PasswordModifyExtendedResponse) ctx.extendedOperation(request);
        Assert.assertNull(response.getGeneratedPassword());
        checkPassword("cn=test2,ou=People,dc=rickyepoderi,dc=es", "changed@test2");
        ctx.close();
    }

    @Test
    public void test3ChangePasswordAdminNoPreviousPassword() throws NamingException, UnknownHostException {
        LdapContext ctx = createConnectionAdmin();
        PasswordModifyExtendedRequest request = new PasswordModifyExtendedRequest(
                "cn=test3,ou=People,dc=rickyepoderi,dc=es", null, "changed@test3");
        PasswordModifyExtendedResponse response = (PasswordModifyExtendedResponse) ctx.extendedOperation(request);
        Assert.assertNull(response.getGeneratedPassword());
        checkPassword("cn=test3,ou=People,dc=rickyepoderi,dc=es", "changed@test3");
        ctx.close();
    }
}
